/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n7_cupiViajes
 * Autor: Equipo Cupi2 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */
package uniandes.cupi2.cupiViajes.interfaz;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Properties;
import java.util.Set;

import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.border.TitledBorder;

import uniandes.cupi2.cupiViajes.excepciones.ClienteTieneReservaException;
import uniandes.cupi2.cupiViajes.excepciones.PersistenciaException;
import uniandes.cupi2.cupiViajes.mundo.Aerolinea;
import uniandes.cupi2.cupiViajes.mundo.ComparadorReservaAdultosCliente;
import uniandes.cupi2.cupiViajes.mundo.ComparadorReservaNombreCliente;
import uniandes.cupi2.cupiViajes.mundo.CriterioOrdenReserva;
import uniandes.cupi2.cupiViajes.mundo.CupiViajes;
import uniandes.cupi2.cupiViajes.mundo.Estado;
import uniandes.cupi2.cupiViajes.mundo.Hotel;
import uniandes.cupi2.cupiViajes.mundo.ReservaViaje;
import uniandes.cupi2.ordenador.AlgoritmoOrdenamiento;
import uniandes.cupi2.ordenador.Ordenador;
import uniandes.cupi2.persistencia.ManejadorArchivosTexto;
import uniandes.cupi2.persistencia.Serializador;

/**
 * Ventana principal de la aplicacin.
 */
public class InterfazCupiViajes extends JFrame
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------
    
    /**
     * Constante que representa la ubicacin del archivo con los datos de los viajes.
     */
    private final static String ARCHIVO_SERIALIZACION = "./data/cupiViajes.data";
    
    /**
     * Constante que representa la ubicacin del archivo con los datos de los hoteles.
     */
    public final static String ARCHIVO_HOTELES = "./data/hoteles.csv";
    
    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------
    
    /**
     * Clase principal del mundo.
     */
    private CupiViajes cupiViajes;
    
    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------
    
    /**
     * Panel con la imagen del encabezado.
     */
    private PanelImagen panelImagen;
    
    /**
     * Panel con la informacin del hotel de la reserva.
     */
    private PanelHotel panelHotel;
    
    /**
     * Panel con la informacin de la reserva.
     */
    private PanelReserva panelReserva;
    
    /**
     * Panel con las opciones de bsqueda y extensin.
     */
    private PanelOpciones panelOpciones;
    
    /**
     * Panel con la lista de reservas hechas.
     */
    private PanelListaReservas panelListaReservas;
    
    private Ordenador<ReservaViaje> ordenadorReservas;
    
    // -----------------------------------------------------------------
    // Constructor
    // -----------------------------------------------------------------
    
    /**
     * Constructor de la ventana principal.<br>
     * <b> post: </b> Construye la ventana principal de la aplicacin.
     */
    public InterfazCupiViajes( )
    {
        cupiViajes = new CupiViajes( );
        
        setLayout( new BorderLayout( ) );
        setTitle( "CupiViajes" );
        setSize( new Dimension( 750, 650 ) );
        //TODO Parte 4A Observe que se usa la propiedad dispose_on_close para que al cerrar el programa se llame al mtodo dispose
        setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
        setLocationRelativeTo( null );
        setResizable( true );
        
        panelImagen = new PanelImagen( );
        add( panelImagen, BorderLayout.NORTH );
        
        JPanel panelAux = new JPanel( );
        panelAux.setLayout( new BorderLayout( ) );
        panelAux.setBorder( new TitledBorder( "Informacin de la reserva" ) );
        
        panelReserva = new PanelReserva(this );
        panelAux.add( panelReserva, BorderLayout.CENTER );
        
        panelHotel = new PanelHotel( this );
        panelAux.add( panelHotel, BorderLayout.SOUTH );
        
        
        add( panelAux, BorderLayout.CENTER );
        
        panelOpciones = new PanelOpciones( this );
        add( panelOpciones, BorderLayout.SOUTH );
        
        panelListaReservas = new PanelListaReservas( this );
        add( panelListaReservas, BorderLayout.WEST );
        
        cargar( );
        ordenadorReservas = new Ordenador<ReservaViaje>( );
    }
    
    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------
    
    /**
     * Carga la informacin de los viajes
     */
    private void cargar( ) 
    {
        File f = new File( ARCHIVO_SERIALIZACION );
        if(f.exists( ))
        {
            try
            {
                //TODO Parte 4A Observe como se llama al serializados
                cupiViajes = Serializador.cargar( ARCHIVO_SERIALIZACION, cupiViajes );
                actualizarListaReservas( );
               
            }
            catch( PersistenciaException e )
            {
                JOptionPane.showMessageDialog( this, e.getMessage( ), "Error", JOptionPane.ERROR_MESSAGE );
            }
        }
        else
        {
            try
            {
              //TODO Parte 4A Observe como se llama al manejador de archivos de texto
                ManejadorArchivosTexto.importarCSV( ARCHIVO_HOTELES, cupiViajes );
            }
            catch( PersistenciaException e )
            {
                JOptionPane.showMessageDialog( this, e.getMessage( ), "Error", JOptionPane.ERROR_MESSAGE );
            }
        }
    }
    
    public void dispose()
    {
        try
        {
            //TODO Parte 4.B Llame al seralizador para que guarde cupiviajes en el archivo de serializacin
            //      Muestre un mensaje indicando que el contenido de cupi viajes se guardo correctamente
            //      Llame al mtodo dispose de la clase padre para que se cierre la ventana
            
        }
        catch( PersistenciaException e )
        {
            int seleccion = JOptionPane.showConfirmDialog( this, e.getMessage( ) + "\nDesea cerrar de todos modos?", "Error", JOptionPane.YES_NO_OPTION );
            if(seleccion == JOptionPane.YES_OPTION)
            {
                // TODO Parte 4.B Llame al mtodo dispose de la clase padre para que se cierre la ventana
               
            }
        }
    }
    
    /**
     * Retorna la lista de hoteles disponibles en el sistema.
     * @return Lista de hoteles disponibles.
     */
    public ArrayList<Hotel> darHoteles( )
    {
        return cupiViajes.darHoteles( );
    }

    /**
     * Actualiza la lista de reservas.
     */
    public void actualizarListaReservas( )
    {
        panelListaReservas.actualizarLista( cupiViajes.darReservas( ) );
    }

    /**
     * Actualiza la informacin de la reserva y del hotel dados por parmetro.
     * @param pReserva Reserva que se va a visualizar. pReserva != null.
     * @param pHotel Hotel que se va a visualizar. pHotel != null.
     */
    public void actualizarInformacion( ReservaViaje pReserva, Hotel pHotel )
    {
        panelReserva.actualizarInformacion( pReserva );
        panelHotel.actualizar( pHotel );
    }

    /**
     * Retorna la lista de hoteles que se encuentran en la ciudad dada por parmetro.
     * @param pCiudad Ciudad de los hoteles buscados. pCiudad != null && pCiudad != "".
     * @return Lista de hoteles que se encuentran en la ciudad dada.
     */
    public ArrayList buscarHotelesPorCiudad( String pCiudad )
    {
        return cupiViajes.buscarHotelesCiudad( pCiudad );
    }
    
    /**
     * Busca la reserva segn el criterio dado por parmetro. <br>
     * <b> post: </b> Se actualiza la informacin de los paneles con la informacin de la reserva. 
     * @param pCriterio Criterio por el que se va a buscar la reserva. pCriterio != null && pCriterio pertenece a { PanelOpciones.CLIENTE, PanelOpciones.CIUDAD, PanelOpciones.MAYOR_PERSONAS, PanelOpciones.MENOR_PERSONAS }.
     */
    public void buscarReserva( String pCriterio )
    {
        if( pCriterio.equals( PanelOpciones.CLIENTE ) )
        {
            ordenadorReservas.ordenar( AlgoritmoOrdenamiento.BURBUJA, cupiViajes.darReservas( ), true, new ComparadorReservaNombreCliente( ) );
            String cliente = JOptionPane.showInputDialog( this, "Por favor indique el nombre del cliente: " );
            if( cliente != null && !cliente.equals( "" ))
            {
                ReservaViaje reserva = cupiViajes.buscarReservaPorClienteBinario( cliente );
                if( reserva != null )
                {
                    panelReserva.actualizarInformacion( reserva );
                    panelHotel.actualizar( reserva.darHotel( ) );
                    panelListaReservas.seleccionarReserva( reserva );
                }
                else
                {
                    JOptionPane.showMessageDialog( this, "No se encontr una reserva con el cliente dado.", "Buscar reserva por cliente", JOptionPane.ERROR_MESSAGE );
                }
            }
            else
            {
                JOptionPane.showMessageDialog( this, "El nombre del cliente no puede ser vaco.", "Buscar reserva por cliente", JOptionPane.ERROR_MESSAGE );
            }
        }
        else if( pCriterio.equals( PanelOpciones.CIUDAD ) )
        {
            String ciudad = JOptionPane.showInputDialog( this, "Por favor indique el nombre de la ciudad: " );
            if( ciudad != null && !ciudad.equals( "" ) )
            {
                ReservaViaje reserva = cupiViajes.buscarReservaPorCiudad( ciudad );
                if( reserva != null )
                {
                    panelReserva.actualizarInformacion( reserva );
                    panelHotel.actualizar( reserva.darHotel( ) );
                    panelListaReservas.seleccionarReserva( reserva );
                }
                else
                {
                    JOptionPane.showMessageDialog( this, "No se encontr una reserva con la ciudad dada.", "Buscar reserva por ciudad", JOptionPane.ERROR_MESSAGE );
                }
            }
            else
            {
                JOptionPane.showMessageDialog( this, "El nombre de la ciudad no puede ser vaca.", "Buscar reserva por ciudad", JOptionPane.ERROR_MESSAGE );
            }
        }
        else if( pCriterio.equals( PanelOpciones.AEROLINEA ) )
        {
            Aerolinea[] opciones = Aerolinea.values( );
            JComboBox comboOpciones = new JComboBox( opciones );
            JOptionPane.showMessageDialog( null, comboOpciones, "Por qu aerolnea desea buscar la reserva?", JOptionPane.QUESTION_MESSAGE );
            Aerolinea aerolinea = ( Aerolinea )comboOpciones.getSelectedItem( );
            if( aerolinea != null )
            {
                ArrayList<ReservaViaje> listaReservas = cupiViajes.buscarReservasAerolinea( aerolinea );
                panelListaReservas.actualizarLista( listaReservas );
            }
        }
        else if( pCriterio.equals( PanelOpciones.MAYOR_PERSONAS ) )
        {
            ReservaViaje reserva = cupiViajes.buscarReservaMasPersonas( );
            if( reserva != null )
            {
                panelReserva.actualizarInformacion( reserva );
                panelHotel.actualizar( reserva.darHotel( ) );
                panelListaReservas.seleccionarReserva( reserva );
            }
            else
            {
                JOptionPane.showMessageDialog( this, "No hay reservas en el sistema.", "Buscar reserva por ciudad", JOptionPane.ERROR_MESSAGE );
            }
        }
        else if( pCriterio.equals( PanelOpciones.MENOR_PERSONAS ) )
        {
            ReservaViaje reserva = cupiViajes.buscarReservaMenosPersonas( );
            if( reserva != null )
            {
                panelReserva.actualizarInformacion( reserva );
                panelHotel.actualizar( reserva.darHotel( ) );
                panelListaReservas.seleccionarReserva( reserva );
            }
            else
            {
                JOptionPane.showMessageDialog( this, "No hay reservas en el sistema.", "Buscar reserva por ciudad", JOptionPane.ERROR_MESSAGE );
            }
        }
    }

    /**
     * Guarda una reserva en el sistema con los valores dados por parmetro.
     * @param pHotel Hotel que se va a reservar. pHotel != null.
     * @param pFechaLlegada fecha de llegada al hotel que se va a reservar. pDiaLlegada !=null
     * @param pCliente Cliente responsable de la reserva. pCliente != null && pCliente != "".
     * @param pCantidadAdultos Cantidad de adultos que van a viajar al hotel que se va a reservar. pCantidadAdultos >= 1 && pCantidadAdultos <= 6.
     * @param pCantidadNinios Cantidad de nios que van a viajar al hotel que se va a reservar. pCantidadNinios >= 0 && pCantidadNinios <= 4.
     * @param pNochesEstadia Cantidad de noches de estada en el hotel que se va a reservar. pNochesEstadia >= 1.
     * @param pAerolinea Aerolnea que se va a reservar. pAerolinea != "" && pAerolinea pertenece a {ReservaViaje.AVIANCA, ReservaViaje.LAN, ReservaViaje.VIVA_COLOMBIA,
     *        ReservaViaje.SATENA, ReservaViaje.JETBLUE, ReservaViaje.IBERIA, ReservaViaje.AIR_FRANCE}.
     */
    public void guardarReserva( Hotel pHotel, Date pFechaLlegada, String pCedula, String pCliente, int pCantidadAdultos, int pCantidadNinios, int pNochesEstadia, Aerolinea pAerolinea )
    {
        try
        {
            cupiViajes.agregarReserva( pHotel, pFechaLlegada, pCedula, pCliente, pCantidadAdultos, pCantidadNinios, pNochesEstadia, pAerolinea );
            actualizarListaReservas( );
        }
        catch( ClienteTieneReservaException e )
        {
            //TODO Parte 2.d Muestre un dialogo con el mensaje de la excepcin y la cdula que viene guarada en ella
            
        }
        
        
    }
    
    /**
     * Crea y visualiza el dilogo para guardar una reserva.
     */
    public void iniciarDialogoGuardarReserva( )
    {
        DialogoAgregarReserva dialogo = new DialogoAgregarReserva( this );
        dialogo.setVisible( true );
    }
    
    /**
     * Crea y visualiza el dilogo para seleccionar el hotel de la reserva.
     * @param pDialogoReserva Referencia al dilogo de reserva. pDialogoReserva != null.
     */
    public void iniciarDialogoSeleccionarHotel( DialogoAgregarReserva pDialogoReserva )
    {
        DialogoSeleccionHotel dialogo = new DialogoSeleccionHotel( this, pDialogoReserva );
        dialogo.inicializarCiudades( cupiViajes.darHoteles( ) );
        dialogo.setVisible( true );
    }
    
    /**
     * Ordena la lista de reservas segn el criterio dado por parmetro. <br>
     * <b> post: </b> La lista de reservas qued ordenada segn el criterio.
     */
    public void ordenarLista( )
    {
        CriterioOrdenReserva[] opciones = CriterioOrdenReserva.values( );
        JComboBox comboOpciones = new JComboBox( opciones );
        JOptionPane.showMessageDialog( null, comboOpciones, "Segn que criterio ordenar la lista?", JOptionPane.QUESTION_MESSAGE );
        CriterioOrdenReserva opcion = ( CriterioOrdenReserva )comboOpciones.getSelectedItem( );

        AlgoritmoOrdenamiento[] algoritmos = AlgoritmoOrdenamiento.values( );
        JComboBox comboAlgoritmos = new JComboBox( algoritmos );
        JOptionPane.showMessageDialog( null, comboAlgoritmos, "Cmo desea ordenar la lista?", JOptionPane.QUESTION_MESSAGE );
        AlgoritmoOrdenamiento algoritmo = ( AlgoritmoOrdenamiento )comboAlgoritmos.getSelectedItem( );
        
        
        if( opcion != null && algoritmo !=null)
        {
            ordenadorReservas.ordenar( algoritmo, cupiViajes.darReservas( ), opcion.esAscendnete( ), opcion.darComparador( ) );
            actualizarListaReservas( );
        }
    }
    
    /**
     * Cambia ele stado de una reserva
     * @param cedula la cdula asociada a la reserva. cedula != null && cedual != ""
     * @param estado el nuevo estado de la reserva. estado !=null
     */
    public void cambiarEstadoReserva( String cedula, Estado estado )
    {
        cupiViajes.cambiarEstadoReserva(cedula, estado);
        JOptionPane.showMessageDialog( this, "Se ha cambiado el estado de la reserva", "Estado", JOptionPane.INFORMATION_MESSAGE );
        
    }

    /**
     * Genera el reporte de reservas
     */
    public void generarReporte( )
    {
        JFileChooser chooser = new JFileChooser( "./data" );
        chooser.setDialogTitle( "Generar reporte" );
        int returnVal = chooser.showSaveDialog( this );
        if( returnVal == JFileChooser.APPROVE_OPTION )
        {
            File arch = chooser.getSelectedFile( );
            try
            {
                ManejadorArchivosTexto.generarReporte( arch.getAbsolutePath( ), cupiViajes );
               JOptionPane.showMessageDialog( this, "Reporte generado.", "Generar reporte", JOptionPane.INFORMATION_MESSAGE );
            }
            catch( PersistenciaException e )
            {
                JOptionPane.showMessageDialog( this, e.getMessage( ), "Generar reporte", JOptionPane.ERROR_MESSAGE );
            }
        }
        
    }
    
    // -----------------------------------------------------------------
    // Puntos de Extensin
    // -----------------------------------------------------------------

    /**
     * Mtodo para la extensin 1.
     */
    public void reqFuncOpcion1( )
    {
        String resultado = cupiViajes.metodo1( );
        JOptionPane.showMessageDialog( this, resultado, "Respuesta", JOptionPane.INFORMATION_MESSAGE );
    }

    /**
     * Mtodo para la extensin 2.
     */
    public void reqFuncOpcion2( )
    {
        String resultado = cupiViajes.metodo2( );
        JOptionPane.showMessageDialog( this, resultado, "Respuesta", JOptionPane.INFORMATION_MESSAGE );
    }

    
    // -----------------------------------------------------------------
    // Main
    // -----------------------------------------------------------------

    /**
     * Este mtodo ejecuta la aplicacin, creando una nueva interfaz.
     * @param pArgs Argumentos para la ejecucin de la aplicacin. pArgs != null.
     */
    public static void main( String[] pArgs )
    {
        InterfazCupiViajes interfaz = new InterfazCupiViajes( );
        interfaz.setVisible( true );

    }

   

}
