/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n7_cupiViajes
 * Autor: Equipo Cupi2 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */
package uniandes.cupi2.cupiViajes.mundo;

import java.io.Serializable;

/**
 * Clase que representa un hotel. <br>
 * <b> inv: </b> <br>
 * nombre != null && nombre != "". <br>
 * ciudad != null && ciudad != "". <br>
 * estrellas >= 0 && estrellas <= 5. <br>
 * costoNoche > 0. <br>
 */
public class Hotel
{
    // -------------------------------------------------------------
    // Atributos
    // -------------------------------------------------------------

    /**
     * Nombre del hotel.
     */
    private String nombre;

    /**
     * Ciudad en la que se encuentra el hotel.
     */
    private String ciudad;

    /**
     * Cantidad de estrellas del hotel.
     */
    private int estrellas;

    /**
     * Costo por noche de estada en el hotel.
     */
    private double costoNoche;
    
    /**
     * Ruta de la imagen del hotel.
     */
    private String rutaImagen;

    // -------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------

    /**
     * Inicializa el hotel con los valores dados por parmetro. <br>
     * <b> post: </b> Los atributos nombre, ciudad, estrellas, costoNoche y rutaImagen fueron inicializados con los valores dados por parmetro.
     * @param pNombre Nombre del hotel. pNombre != null && pNombre != "".
     * @param pCiudad Ciudad en la que se encuentra el hotel. pCiudad != null && pCiuidad != "".
     * @param pEstrellas Estrellas del hotel. pEstrellas >= 0 && pEstrellas <= 5.
     * @param pCostoNoche Costo por noche de estada en el hotel. pCostoNoche > 0.
     * @param pRutaImagen Ruta de la imagen del hotel. pRutaImagen != null && pRutaImagen != "".
     */
    public Hotel( String pNombre, String pCiudad, int pEstrellas, double pCostoNoche, String pRutaImagen )
    {
        nombre = pNombre;
        ciudad = pCiudad;
        estrellas = pEstrellas;
        costoNoche = pCostoNoche;
        rutaImagen = pRutaImagen;
        verificarInvariante( );
    }

    // -------------------------------------------------------------
    // Mtodos
    // -------------------------------------------------------------

    /**
     * Retorna el nombre del hotel.
     * @return Nombre del hotel.
     */
    public String darNombre( )
    {
        return nombre;
    }

    /**
     * Retorna la ciudad en la que se encuentra el hotel.
     * @return Ciudad en la que se encuentra el hotel.
     */
    public String darCiudad( )
    {
        return ciudad;
    }

    /**
     * Retorna la cantidad de estrellas del hotel.
     * @return Cantidad de estrellas del hotel.
     */
    public int darEstrellas( )
    {
        return estrellas;
    }

    /**
     * Retorna el costo por noche de estada en el hotel.
     * @return Costo por noche de estada.
     */
    public double darCostoNoche( )
    {
        return costoNoche;
    }
    
    /**
     * Retorna la ruta de la imagen del hotel.
     * @return Ruta de la imagen del hotel.
     */
    public String darRutaImagen( )
    {
        return rutaImagen;
    }

    /**
     * Retorna una cadena con el nombre del hotel.
     * @return Representacin del hotel en una cadena de caracteres.
     */
    public String toString( )
    {
        return nombre;
    }

    // -----------------------------------------------------------------
    // Invariante
    // -----------------------------------------------------------------

    /**
     * Verifica el invariante de la clase. <br>
     * <b> inv: </b> <br>
     * nombre != null && nombre != "". <br>
     * ciudad != null && ciudad != "". <br>
     * estrellas >= 0 && estrellas <= 5. <br>
     * costoNoche > 0. <br>
     */
    private void verificarInvariante( )
    {
        assert ( nombre != null && nombre != "" ) : "El nombre del hotel no puede ser nulo o vaco.";
        assert ( ciudad != null && ciudad != "" ) : "La ciudad del hotel no puede ser nula o vaca.";
        assert ( estrellas >= 0 && estrellas <= 5 ) : "El hotel debe tener de 0 a 5 estrellas.";
        assert ( costoNoche > 0 ) : "El costo por noche del hotel debe ser mayor a cero.";
    }

}
